from django.apps import AppConfig
from django.core.management import call_command
from django.db.models.signals import post_migrate


def populate_data(sender, **kwargs):
    call_command("populate_countries", verbosity=1)


class CountryConfig(AppConfig):
    name = 'country'

    def ready(self):
        post_migrate.connect(populate_data, sender=self)



from django.contrib.postgres.fields import JSONField
from django.db import models
from model_utils.models import TimeStampedModel


class Country(TimeStampedModel):
    name = models.CharField(max_length=255)
    code = models.CharField(max_length=45, unique=True)
    language = models.CharField(max_length=45)
    extra = JSONField(blank=True, null=True)
    currency = models.ForeignKey(
        "currency.Currency",
        on_delete=models.CASCADE,
        related_name="countries",
    )

    class Meta:
        db_table = "country"
        verbose_name_plural = "countries"

    def __str__(self):
        return self.name



from django.contrib import admin
from .models import Currency


@admin.register(Currency)
class CurrencyAdmin(admin.ModelAdmin):
    list_display = (
        "name",
        "abbreviation",
        "currency_symbol",
        "currency_format",
        "number_format",
    )



from django.contrib import admin
from .models import Country


@admin.register(Country)
class CountryAdmin(admin.ModelAdmin):
    list_display = (
        "name",
        "code",
        "language",
        "extra",
        "currency"
    )
    raw_id_fields = ("currency",)



from django.conf import settings
from django.core.management import call_command
from django.core.management.base import BaseCommand
from os import path


class Command(BaseCommand):
    help = "Populates data"

    def handle(self, *args, **options):
        fixture_path = path.join(path.dirname(
            path.dirname(
                path.dirname(
                    path.abspath(__file__)
                )
            )
        ), "fixtures/")
        settings.FIXTURE_DIRS = (fixture_path,)
        call_command("loaddata", "country", verbosity=1)



from babel.numbers import format_decimal
from django.db import models
from model_utils.models import TimeStampedModel


class CurrencyManager(models.Manager):

    def get_by_natural_key(self, abbreviation):
        return self.get(abbreviation=abbreviation)

    def format_money(self, money, currency_abbreviation="PEN"):
        currrency = self.get_by_natural_key(currency_abbreviation)
        country = currency.countries.first()
        amount_formated = format_decimal(
            money,
            format=currency.number_format,
            locale="{}_{}".format(country.language, country.code),
        )
        return currency.currency_format.format(amount_formated)


class Currency(TimeStampedModel):
    abbreviation = models.CharField(max_length=45, unique=True)
    name = models.CharField(max_length=255)
    currency_symbol = models.CharField(max_length=5)
    currency_format = models.CharField(max_length=45)
    number_format = models.CharField(max_length=45)

    objects = CurrencyManager()
    
    class Meta:
        db_table = "currency"
        verbose_name_plural = "currencies"

    def __str__(self):
        return self.name
